@@ -74,7 +74,8 @@ gconf_editor_window_go_to (GConfEditorWindow *window,
 {
 	GtkTreePath *path, *child_path;
 	gint depth;
-	gint i;
+	gint i, nchildren;
+	char *key;
 
 	g_return_if_fail (location != NULL);
 
@@ -114,6 +115,33 @@ gconf_editor_window_go_to (GConfEditorWindow *window,
 	gtk_tree_view_scroll_to_cell (GTK_TREE_VIEW (window->tree_view),
 				      path, NULL, TRUE, 0.5, 0.5);
 	gtk_tree_path_free (path);
+
+	/* check for key name in the list model */
+	key = rindex (location, '/');
+	if (!key || strlen (key) < 2)
+		return;
+	key++;
+	nchildren = gtk_tree_model_iter_n_children (window->list_model, NULL);
+        for (i = 0; i < nchildren; i++) {
+		GtkTreeIter iter;
+		char *name;
+		gtk_tree_model_iter_nth_child (window->sorted_list_model,
+						&iter, NULL, i);
+
+		gtk_tree_model_get (window->sorted_list_model, &iter,
+				    GCONF_LIST_MODEL_KEY_NAME_COLUMN, &name, -1);
+		if (strcmp (key, name) == 0) {
+			path = gtk_tree_model_get_path (window->sorted_list_model, &iter);
+			gtk_tree_selection_select_path (gtk_tree_view_get_selection (GTK_TREE_VIEW (window->list_view)),
+                        				path);
+			gtk_tree_view_scroll_to_cell (GTK_TREE_VIEW (window->list_view),
+						      path, NULL, TRUE, 0.5, 0.5);
+			gtk_tree_path_free (path);
+			g_free (name);
+			return;
+		}
+		g_free (name);
+	}
 }
 
 static void
@@ -169,6 +197,61 @@ gconf_editor_window_add_bookmark (GtkAction *action, GtkWidget *callback_data)
 	g_free (path);
 }
 
+static void
+gconf_editor_window_recents_init (void)
+{
+	GConfClient *client = gconf_client_get_default ();
+
+	gconf_client_add_dir (client, "/apps/gconf-editor/recents",
+			      GCONF_CLIENT_PRELOAD_ONELEVEL, NULL);
+
+	g_object_unref (client);
+}
+
+static void
+gconf_add_recent_key (GConfEditorWindow *window, const char *key)
+{
+	GSList *list, *tmp;
+	gboolean add = TRUE;
+
+	/* Get the old list and then set it */
+	list = gconf_client_get_list (window->client,
+				      "/apps/gconf-editor/recents", GCONF_VALUE_STRING, NULL);
+
+	/* Check that the recent hasn't been added already */
+	for (tmp = list; tmp; tmp = tmp->next) {
+		if (strcmp ((gchar*) tmp->data, key) == 0)
+			add = FALSE;
+	}
+
+	/* Append the new key */
+	if (add == TRUE) {
+		/* Keep list on a fixed size, remove the last */
+		if (g_slist_length (list) > RECENT_LIST_MAX_SIZE - 1) {
+			tmp = g_slist_last (list);
+			list = g_slist_delete_link (list, tmp);
+		}
+
+		list = g_slist_prepend (list, g_strdup (key));
+
+		gconf_client_set_list (window->client,
+				       "/apps/gconf-editor/recents", GCONF_VALUE_STRING, list, NULL);
+
+		if (GTK_WIDGET_VISIBLE (GTK_WIDGET (window->output_window)) &&
+	    				window->output_window_type == GCONF_EDITOR_WINDOW_OUTPUT_WINDOW_RECENTS) 
+			gedit_output_window_prepend_line (GEDIT_OUTPUT_WINDOW (window->output_window),
+							  (gchar*) key, TRUE);
+	}
+
+
+	for (tmp = list; tmp; tmp = tmp->next) {
+		g_free (tmp->data);
+	}
+
+	g_slist_free (list);
+}
+
+
 static void
 gconf_editor_window_edit_bookmarks (GtkAction *action, GtkWidget *callback_data)
 {
@@ -200,6 +283,25 @@ gconf_editor_window_search (GtkAction *action, GtkWidget *callback_data)
 	gtk_widget_show (dialog);
 }
 
+static void
+gconf_editor_show_recent_keys (GtkAction *action, GtkWidget *window)
+{
+	GSList *list, *tmp;
+	gedit_output_window_clear (GEDIT_OUTPUT_WINDOW (GCONF_EDITOR_WINDOW (window)->output_window));
+	list = gconf_client_get_list (GCONF_EDITOR_WINDOW (window)->client,
+                                     "/apps/gconf-editor/recents", GCONF_VALUE_STRING, NULL);
+	for (tmp = list; tmp; tmp = tmp->next) {
+		gedit_output_window_append_line (GEDIT_OUTPUT_WINDOW (GCONF_EDITOR_WINDOW (window)->output_window),
+						 (gchar*) tmp->data, FALSE);
+		g_free (tmp->data);
+	}
+
+	GCONF_EDITOR_WINDOW (window)->output_window_type = GCONF_EDITOR_WINDOW_OUTPUT_WINDOW_RECENTS;
+	gtk_widget_show (GTK_WIDGET (GCONF_EDITOR_WINDOW (window)->output_window));
+	
+	g_slist_free (list);
+}
+
 static void
 gconf_editor_window_new_window (GtkAction *action, GtkWidget *callback_data)
 {
@@ -360,6 +462,7 @@ gconf_editor_new_key_response (GtkDialog *editor,
 		value = gconf_key_editor_get_value (GCONF_KEY_EDITOR (editor));
 
 		gconf_client_set (window->client, full_path, value, NULL);
+		gconf_add_recent_key (window, full_path);
 
 		if(value)
 		  gconf_value_free (value);
@@ -415,8 +518,10 @@ gconf_editor_edit_key_response (GtkDialog *editor,
 		g_assert (gconf_valid_key (path, NULL));
 
 		/* if not writable we weren't allowed to change things anyway */
-		if (gconf_client_key_is_writable (window->client, path, &error))
+		if (gconf_client_key_is_writable (window->client, path, &error)) {
 			gconf_client_set (window->client, path, value, &error);
+			gconf_add_recent_key (window, path);
+		}
 
 		gconf_value_free (value);
 
@@ -626,28 +731,30 @@ static GtkActionEntry entries[] = {
 	{ "BookmarksMenu", NULL, N_("_Bookmarks"), NULL, NULL, NULL },
 	{ "HelpMenu", NULL, N_("_Help"), NULL, NULL, NULL },
 
-	{ "NewWindow", GTK_STOCK_NEW, N_("New _settings window"), "<control>S", 
+	{ "NewWindow", GTK_STOCK_NEW, N_("New _Settings Window"), "<control>S", 
 	  N_("Open a new gconf editor window editing current settings"), 
 	  G_CALLBACK (gconf_editor_window_new_window) },
-	{ "NewDefaultsWindow", GTK_STOCK_NEW, N_("New _defaults window"), "<control>D", 
+	{ "NewDefaultsWindow", GTK_STOCK_NEW, N_("New _Defaults Window"), "<control>D", 
           N_("Open a new gconf editor window editing system default settings"), 
 	  G_CALLBACK (gconf_editor_window_new_defaults_window) },
-	{ "NewMandatoryWindow", GTK_STOCK_NEW, N_("New _mandatory window"), "<control>M", 
+	{ "NewMandatoryWindow", GTK_STOCK_NEW, N_("New _Mandatory Window"), "<control>M", 
           N_("Open a new gconf editor window editing system mandatory settings"), 
 	  G_CALLBACK (gconf_editor_window_new_mandatory_window) },
-	{ "CloseWindow", GTK_STOCK_CLOSE, N_("_Close window"), "<control>W", N_("Close this window"), 
+	{ "CloseWindow", GTK_STOCK_CLOSE, N_("_Close Window"), "<control>W", N_("Close this window"), 
 	  G_CALLBACK (gconf_editor_window_close_window) },
 	{ "QuitGConfEditor", GTK_STOCK_QUIT, N_("_Quit"), "<control>Q", N_("Quit the gconf editor"), 
 	  G_CALLBACK (gtk_main_quit) },
 	
-	{ "CopyKeyName", GTK_STOCK_COPY, N_("_Copy key name"), "<control>C", N_("Copy the name of the selected key"), 
+	{ "CopyKeyName", GTK_STOCK_COPY, N_("_Copy Key Name"), "<control>C", N_("Copy the name of the selected key"), 
 	  G_CALLBACK (gconf_editor_window_copy_key_name) },
-	{ "Find", GTK_STOCK_FIND, N_("Find"), "<control>F", N_("Find patterns in keys and values"), 
+	{ "Find", GTK_STOCK_FIND, N_("Find..."), "<control>F", N_("Find patterns in keys and values"), 
 	  G_CALLBACK (gconf_editor_window_search) },
+	{ "RecentKeys", NULL, N_("List Recent keys"), "<control>R", N_("Show recently modified keys"), 
+	  G_CALLBACK (gconf_editor_show_recent_keys) },
 
-	{ "AddBookmark", NULL, N_("_Add bookmark"), NULL, N_("Add a bookmark to the selected directory"), 
+	{ "AddBookmark", NULL, N_("_Add Bookmark"), NULL, N_("Add a bookmark to the selected directory"), 
 	  G_CALLBACK (gconf_editor_window_add_bookmark) },
-	{ "EditBookmarks", NULL, N_("_Edit bookmarks"), NULL, N_("Edit the bookmarks"), 
+	{ "EditBookmarks", NULL, N_("_Edit Bookmarks"), NULL, N_("Edit the bookmarks"), 
 	G_CALLBACK (gconf_editor_window_edit_bookmarks) },
 	
 	{ "HelpContents", GTK_STOCK_HELP, N_("_Contents"), "F1", N_("Open the help contents for the gconf editor"), 
@@ -655,15 +762,15 @@ static GtkActionEntry entries[] = {
 	{ "AboutAction", GNOME_STOCK_ABOUT, N_("_About"), NULL, N_("Show the about dialog for the gconf editor"), 
 	  G_CALLBACK (gconf_editor_window_about_window) },
 
-	{ "NewKey", GTK_STOCK_NEW, N_("_New key..."), "<control>N", N_("Create a new key"),
+	{ "NewKey", GTK_STOCK_NEW, N_("_New Key..."), "<control>N", N_("Create a new key"),
 	  G_CALLBACK (gconf_editor_popup_window_new_key) },
-	{ "EditKey", NULL, N_("_Edit key..."), NULL, N_("Edit the selected key"),
+	{ "EditKey", NULL, N_("_Edit Key..."), NULL, N_("Edit the selected key"),
 	  G_CALLBACK (gconf_editor_popup_window_edit_key) },
-	{ "UnsetKey", GTK_STOCK_DELETE, N_("_Unset key..."), NULL, N_("Unset the selected key"),
+	{ "UnsetKey", GTK_STOCK_DELETE, N_("_Unset Key..."), NULL, N_("Unset the selected key"),
 	  G_CALLBACK (gconf_editor_popup_window_unset_key) },
-	{ "DefaultKey", NULL, N_("Set as _default"), NULL, N_("Set the selected key to be the default"),
+	{ "DefaultKey", NULL, N_("Set as _Default"), NULL, N_("Set the selected key to be the default"),
 	  G_CALLBACK (gconf_editor_popup_window_set_as_default) },
-	{ "MandatoryKey", NULL, N_("Set as _mandatory"), NULL, N_("Set the selected key to the mandatory"),
+	{ "MandatoryKey", NULL, N_("Set as _Mandatory"), NULL, N_("Set the selected key to the mandatory"),
 	  G_CALLBACK (gconf_editor_popup_window_set_as_mandatory) }	
 };
 
@@ -680,6 +787,9 @@ static const char *ui_description =
 	"		</menu>"
 	"		<menu action='EditMenu'>"
 	"			<menuitem action='CopyKeyName'/>"
+	"			<separator/>"
+	"			<menuitem action='RecentKeys'/>"
+	"			<separator/>"
 	"			<menuitem action='Find'/>"
 	"		</menu>"	
 	"		<menu action='BookmarksMenu'>"
@@ -711,6 +821,7 @@ static gchar *image_menu_items_paths[] = {
 	"/GConfEditorMenu/FileMenu/QuitGConfEditor",
 
 	"/GConfEditorMenu/EditMenu/CopyKeyName",
+	"/GConfEditorMenu/EditMenu/RecentKeys",
 	"/GConfEditorMenu/EditMenu/Find",
 
 	"/GConfEditorMenu/HelpMenu/HelpContents",
@@ -832,8 +943,10 @@ gconf_editor_gconf_value_changed (GConfCellRenderer *cell, const gchar *path_str
 
 	/* We need to check this because the changed signal could come from an old
 	 * cell in another list_model */ 
-	if (value->type == new_value->type)
+	if (value->type == new_value->type) {
 		gconf_client_set (window->client, key, new_value, NULL);
+		gconf_add_recent_key (window, key);
+	}
 
 	g_free (value);
 	g_free (key);
@@ -1253,20 +1366,28 @@ gconf_editor_window_init (GConfEditorWindow *window)
 				      gtk_ui_manager_get_widget (window->ui_manager, "/GConfEditorMenu/BookmarksMenu/EditBookmarks"));
 
 	/* Create content area */
+	vpaned = gtk_vpaned_new ();
+	gtk_paned_set_position (GTK_PANED (vpaned), 400);
+	gtk_box_pack_start (GTK_BOX (vbox), vpaned, TRUE, TRUE, 0);
+
 	hpaned = gtk_hpaned_new ();
 	gtk_paned_set_position (GTK_PANED (hpaned), 250);
-	gtk_box_pack_start (GTK_BOX (vbox), hpaned, TRUE, TRUE, 0);
+
+	gtk_paned_pack1 (GTK_PANED (vpaned), hpaned, FALSE, FALSE);
 
 	/* Create ouput window */
 	window->output_window = gedit_output_window_new ();
+	window->output_window_type = GCONF_EDITOR_WINDOW_OUTPUT_WINDOW_NONE;
 	gedit_output_window_set_select_multiple (GEDIT_OUTPUT_WINDOW (window->output_window),
 						 GTK_SELECTION_SINGLE);
-	gtk_box_pack_start (GTK_BOX (vbox), window->output_window, FALSE, FALSE, 0);
+	gtk_paned_pack2 (GTK_PANED (vpaned), window->output_window, FALSE, FALSE);
 	g_signal_connect (G_OBJECT (window->output_window), "close_requested",
 			  G_CALLBACK (gconf_editor_window_close_output_window), window);
 
 	g_signal_connect (G_OBJECT (window->output_window), "selection_changed",
 			  G_CALLBACK (gconf_editor_window_output_window_changed), window);
+
+	gconf_editor_window_recents_init ();
 	
 	/* Create status bar */
 	window->statusbar = gtk_statusbar_new ();
